/**@file
 * CopySource.m
 * @brief Copy source class.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/19.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "CopySource.h"
#import "ViewController.h"
#import "FileDisplayCollection.h"
#import "ShsslClient.h"
#import "ShsslAppDelegate.h"
#import "FileDropTableView.h"

@implementation CopySource

// properties.
@synthesize m_directory;
@synthesize m_fileName;
@synthesize m_cut;
@synthesize m_srcController;

/**
 * contructor.
 * @param[in] fileName file name.
 * @param[in] isCut cut flag.
 * @param[in] controller source conroller.
 * @return self;
 */
- (id)initWithFileName:(NSString *)fileName isCut:(BOOL)isCut sourceController:(id)controller
{
	self = [super init];
	if (self) {
		m_directory = nil;
		m_fileName = nil;
		// divide into directory and file.
		const NSRange rng = [fileName rangeOfString:@"/" options:NSBackwardsSearch];
		if (rng.location == 0) {
			// root.
			[self setDirectory:@"/"];
		}
		else {
			// not a root.
			[self setDirectory:[fileName substringToIndex:rng.location]];
		}
		[self setFileName:[fileName substringFromIndex:rng.location + 1]];

		m_cut = isCut;
		m_srcController = controller;
	}
	return self;
}

/**
 * copy.
 * @param[in] targetController copy destination controller.
 * @return YES if success, NO if failed.
 * @throw NSException.
 */
- (BOOL)copyTo:(id)targetController
{
	BOOL ret = NO;

	ShsslClient *const client = [m_srcController client];	// source controller client.
	NSString *const clientDir = [client currentDirectory];	// backup current directory for later restore.
	NSString *const controllerDir = [[m_srcController fileDisplayController] currentDirectory];
	//NSIndexSet *const indices = [[m_srcController fileView] selectedRowIndexes];
	@try {
		if (m_srcController != targetController) {
			[m_srcController setCanClose:NO];
		}
		[clientDir retain];
		[controllerDir retain];

		FileDisplay *const fd = [m_srcController listDir:m_directory file:m_fileName];
		if (fd == nil) {
			@throw [NSException exceptionWithName:@"copyTo" reason:@"source file not found" userInfo:nil];
		}

		// parent directory.
		FileDisplayCollection *const dir = [m_srcController findDirectory:m_directory];

		if ([fd isDirectory] == NO) {
			// physical file.
			[client setCurrentDirectory:[dir fullPath]];
			NSMutableData *const content = [NSMutableData data];
			if ([client retr:m_fileName content:content] == NO) {
				@throw [NSException exceptionWithName:@"copyTo" reason:@"RETR failed" userInfo:nil];
			}

			// stor.
			if ([targetController isCanceled] == NO) {
				if ([targetController storeFile:m_fileName directory:m_directory content:content] == NO) {
					@throw [NSException exceptionWithName:@"copyTo" reason:@"STOR failed" userInfo:nil];
				}
			}

			// if cut.
			if ((m_cut) && ([targetController isCanceled] == NO)) {
				// RM.
				[client setCurrentDirectory:[dir fullPath]];
				if ([client rm:m_fileName] == NO) {
					NSString *const msg = [NSString stringWithFormat:NSLocalizedString(@"remove failed", @"remove failed"), m_fileName];
					@throw [NSException exceptionWithName:@"copyTo" reason:msg userInfo:nil];
				}
			}
		}
		else {
			// directory.
			NSString *const fullPath = [m_directory stringByAppendingPathComponent:m_fileName];
			FileDisplayCollection *const fd = [m_srcController findDirectory:fullPath];
			if ([targetController copyDirectory:fd srcController:m_srcController] == NO) {
				@throw [NSException exceptionWithName:@"copyTo" reason:@"unknown error" userInfo:nil];
			}

			// if cut.
			if ((m_cut) && ([targetController isCanceled] == NO)) {
				// RMD.
				[client setCurrentDirectory:m_directory];
				if ([m_srcController rmvFile:m_fileName isDirectory:YES] == NO) {
					NSString *const msg = [NSString stringWithFormat:NSLocalizedString(@"remove failed", @"remove failed"), m_fileName];
					@throw [NSException exceptionWithName:@"copyTo" reason:msg userInfo:nil];
				}
			}
		}

		// if cut.
		if (m_cut) {
			// empty copy.
			id delegate = [NSApp delegate];
			[delegate emptyCopy];
		}
		// success.
		ret = YES;
	}
	@catch (NSException *e) {
		@throw e;
	}
	@finally {
		[client setCurrentDirectory:clientDir];
		if (m_srcController != targetController) {
			id firstFd = [m_srcController findDirectory:controllerDir];
			[m_srcController silentDirectorySelectionHasChanged:firstFd];
			[m_srcController setCanClose:YES];
		}
		//[indices release];
		[controllerDir retain];
		[clientDir release];
	}
	return ret;
}

/**
 * get last path component.
 * @return file name.
 */
- (NSString *)lastPathComponent
{
	return m_fileName;
}

// destructor.
- (void)dealloc
{
	[m_directory release];
	[m_fileName release];
	[super dealloc];
}

@end
