/**@file
 * Encoder.m
 * @brief Encodert class.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/08.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "Encoder.h"
#import "MyUtil.h"

// constants.
static const unsigned char g_badBetterEmpty[] = { };
static const unsigned char g_bad3[] = { 0xEF, 0xBD, 0x9E, 0xEF, 0xBC, 0x8D, };
static const unsigned char g_better3[] = { 0xE3, 0x80, 0x9C, 0xE3, 0x83, 0xBC, };
/**
 * good and better pair struct.
 */
typedef struct tagBAD_BETTER {
	const size_t count;				// count of bad & better pair.
	const unsigned char *bad;		// bad character.
	const unsigned char *better;	// that bad's better character.
} BAD_BETTER;
static const BAD_BETTER g_bad_betterPair[] = {
	{ 0, g_badBetterEmpty, g_badBetterEmpty },	// 1 byte character's
	{ 0, g_badBetterEmpty, g_badBetterEmpty },	// 2 byte character's
	{ 2, g_bad3, g_better3 },					// 3 byte character's
	{ 0, g_badBetterEmpty, g_badBetterEmpty },	// 4 byte character's
	{ 0, g_badBetterEmpty, g_badBetterEmpty },	// 5 byte character's
	{ 0, g_badBetterEmpty, g_badBetterEmpty },	// 6 byte character's
};

/**
 * character converter sruct.
 */
typedef struct tagXML_CHAR_CONV {
	NSString *from;	// &, <, >, *
	NSString *to;	// &amp;, &lt;, &gt;, &quot; &nbsp;
} XML_CHAR_CONV;
// XML chars encode, decode.
static const XML_CHAR_CONV g_conv[] = {{@"&", @"&amp;"}, {@"<", @"&lt;"}, {@">", @"&gt;"}, {@"\"", @"&quot;"}};

@implementation Encoder

/**
 * encode string HTTP request.
 * @param[in] str string to encode.
 * @return encoded string.
 */
+ (NSString *)encodeHttpString:(NSString *)str
{
	const char thoseAscii[] = "#[]!$&’()*+,;:/?@=";
	return [self utf8PercentEscaping:str withThose:thoseAscii];
}

/**
 * UTF8 percent escaping.
 * @param[in] utf8Str original character using as UTF8 string.
 * @param[in] thoseAscii those asciis also convert.
 * @return percent escaped acsii string.
 */
+ (NSString *)utf8PercentEscaping:(NSString *)utf8Str withThose:(const char *)thoseAscii
{
	NSData *data = nil;
	NSMutableString *const ret = [NSMutableString string];
	@try {
		data = [utf8Str dataUsingEncoding:NSUTF8StringEncoding];
		const NSUInteger len = [data length];
		const NSUInteger thoseLen = strlen(thoseAscii);
		NSUInteger index = 0;
		const unsigned char *uc = (const unsigned char *) [data bytes];
		while (index < len) {
			if (*uc < 0x80) {
				// 1 byte character.
				BOOL hitThose = NO;
				for (NSUInteger i = 0; i < thoseLen; i++) {
					if (*uc == (unsigned char) thoseAscii[i]) {
						hitThose = YES;
						break;
					}
				}
				if (hitThose == NO) {
					[ret appendFormat:@"%C", *uc];
				}
				else {
					[ret appendFormat:@"%%%02X", *uc];
				}
				index++;
				uc++;
			}
			else {
				NSUInteger loop = 0;
				const unsigned char *escaped = NULL;
				// multi bytes.
				if ((*uc >= 0x30) && (*uc <= 0xDF)) {
					// 2bytes.
					loop = 2;
				}
				else if ((*uc >= 0xE0) && (*uc <= 0xEF)) {
					// 3bytes.
					loop = 3;
					for (NSUInteger i = 0; i < g_bad_betterPair[2].count; i++) {
						const unsigned char *const badChar = g_bad_betterPair[2].bad + i * 3;
						if ((uc[0] == badChar[0]) && (uc[1] == badChar[1]) && (uc[2] == badChar[2])) {
							escaped = uc + 3;
							uc = g_bad_betterPair[2].better + i * 3;
						}
					}
				}
				else if ((*uc >= 0xF0) && (*uc <= 0xF7)) {
					// 4bytes.
					loop = 4;
				}
				else if ((*uc >= 0xF8) && (*uc <= 0xFB)) {
					// 5bytes.
					loop = 5;
				}
				else {
					// 6bytes.
					loop = 6;
				}
				// hex convert loop.
				for (NSUInteger i = 0; i < loop; i++) {
					[ret appendFormat:@"%%%02X", *uc];
					index++;
					uc++;
				}
				// if badboy has come...
				if (escaped != NULL) {
					uc = escaped;
					escaped = NULL;
				}
			}
		}
	}
	@catch (NSException *e) {
	}
	@finally {
	}
	//NSLog(@"ret = \"%@\"", ret);
	return ret;
}

/**
 * XML string encode.
 * @param fromStr character string before XML convert.
 * @return Character string after XML convert.
 */
+ (NSString *)encodeXmlString:(NSString *)fromStr
{
	NSString *str = [fromStr copy];
	if (str == nil) {
		return @"";
	}
	for (NSUInteger i = 0; i < COUNT_OF(g_conv, XML_CHAR_CONV); i++) {
		str = [[str stringByReplacingOccurrencesOfString:g_conv[i].from withString:g_conv[i].to] copy];
	}
	return str;
}

/**
 * XML string decode.
 * @param fromStr character string after XML convert.
 * @return Character string after XML convert.
 */
+ (NSString *)decodeXmlString:(NSString *)fromStr
{
	NSString *str = [fromStr copy];
	if (str == nil) {
		return @"";
	}
	for (NSInteger i = COUNT_OF(g_conv, XML_CHAR_CONV) - 1; i > -1; i--) {
		str = [str stringByReplacingOccurrencesOfString:g_conv[i].to withString:g_conv[i].from];
	}
	return str;
}

@end
