/**@file
 * FileDisplayCollection.m
 * @brief Collection class for FileDisplay.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/08.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "FileDisplayCollection.h"
#import "ViewController.h"
#import "MyUtil.h"
#import "ShsslClient.h"
#import "FileDropTableView.h"
#import "ShsslAppDelegate.h"

@implementation FileDisplayCollection

// properties.
@synthesize m_controller;
@synthesize m_fullPath;
@synthesize m_identifierTagOnEdit;
@synthesize m_columnValueOnEdit;
@synthesize m_sortKey;
@synthesize m_ascending;
//@synthesize m_dummy;

/**
 * set full path name.
 * @param[in] fullPath new full path name.
 */
- (void)setFullPath:(NSString *)fullPath
{
	const NSRange range = [fullPath rangeOfString:@" -> "];
	NSString *newString;
	if (range.length != 0) {
		newString = [fullPath substringToIndex:range.location];
	}
	else {
		newString = fullPath;
	}

	[m_fullPath release];
	m_fullPath = newString;
	[m_fullPath retain];
}

/**
 * get object count.
 * @return object count.
 */
- (NSUInteger)count
{
	return [m_displays count];
}

/**
 * get directory count.
 * @return directory count.
 */
- (NSUInteger)directoryCount
{
	NSUInteger i = 0;
	for (id item in m_displays) {
		if ([item class] == [FileDisplayCollection class]) {
			i++;
		}
	}
	return i;
}

/**
 * get object.
 * @param[in] index The index of object to get.
 * @return object which corresponds to the index.
 */
- (id)objectAtIndex:(NSUInteger)index
{
	return [m_displays objectAtIndex:index];
}

/**
 * get display object count.
 * @return directory count.
 */
- (NSUInteger)displayCount
{
	return [m_sotedArray count];
}

/**
 * display object at index.
 * @param[in] index The index of object to get.
 * @return object which corresponds to the index.
 */
- (id)displayObjectAtIndex:(NSUInteger)index
{
	return [m_sotedArray objectAtIndex:index];
}

/**
 * get directory.
 * @param[in] index The index of directory to get.
 * @return directory which corresponds to the index.
 */
- (id)directoryAtIndex:(NSUInteger)index
{
	NSUInteger dirCnt = 0;
	for (NSUInteger i = 0; i < [m_displays count]; i++) {
		id item = [m_displays objectAtIndex:i];
		if ([item class] == [FileDisplayCollection class]) {
			if (index == dirCnt) {
				return item;
			}
			dirCnt++;
		}
	}
	return nil;
}

/**
 * remove all objects.
 */
- (void)removeAllObjects
{
	[m_displays removeAllObjects];
}

/**
 * remove one item.
 * @param[in] index item index to remove.
 */
- (void)removeObjectAtIndex:(NSUInteger)index
{
	[m_displays removeObjectAtIndex:index];
}

/**
 * remove one item of name.
 * @param[in] fileName file name to remove.
 */
- (void)removeObjectOfName:(NSString *)fileName
{
	for (NSUInteger i = 0; i < [m_displays count]; i++) {
		id fd = [m_displays objectAtIndex:i];
		if ([fileName compare:[fd fileName]] == NSOrderedSame) {
			[m_displays removeObjectAtIndex:i];
			break;
		}
	}
	for (NSUInteger i = 0; i < [m_sotedArray count]; i++) {
		id fd = [m_sotedArray objectAtIndex:i];
		if ([fileName compare:[fd fileName]] == NSOrderedSame) {
			[m_sotedArray removeObjectAtIndex:i];
			break;
		}
	}
}

/**
 * add display line.
 * @param[in] ftpResult FTP result string.
 * @return self
 */
- (void)addFileDisplay:(NSString *)ftpResult
{
	//NSLog(@"addFileDisplay:%@", self);

	FileDisplay *const fd = [[FileDisplay alloc] initWithFtpResult:ftpResult];	// create new object.
	FileDisplay *const fdOld = [m_invalidates objectForKey:[fd fileName]];	// find from invalidated object.
	if ((fdOld != nil) && ([fd isEqualFile:fdOld])) {
		[m_displays addObject:fdOld];	// use old one for display.
		[m_invalidates removeObjectForKey:[fd fileName]];
	}
	else {
		if ((fdOld != nil) && ([fd isDirectory] == [fdOld isDirectory])) {
			[fdOld copyFrom:fd];
			[m_displays addObject:fdOld];	// use old one for display.
			[m_invalidates removeObjectForKey:[fd fileName]];
		}
		else {
			if ([fd isDirectory]) {
				// directory.
				FileDisplayCollection *const fdc = [FileDisplayCollection fileDisplayCollectionWithFileDisplay:fd];
				[fdc setController:m_controller];

				// setting full path name.
				[fdc setFullPath:[m_fullPath stringByAppendingPathComponent:[fd fileName]]];
				[fdc setPathDictionary:m_pathDictionary];	// set path dictionary.
				[fdc copyChildren:fdOld];
				[fdOld release];	// fdold no use anymore.
				[m_displays addObject:fdc];	// use new one (directory).
				[m_invalidates removeObjectForKey:[fdc fileName]];
				[fdc release];
			}
			else {
				// file.
				[m_displays addObject:fd];	// use new one.
				[m_invalidates removeObjectForKey:[fd fileName]];
			}
		}
	}
	
	//NSLog(@"file display = %@", fd);
	[fd release];
}

/**
 * before add files.
 */
- (void)invalidate
{
	for (FileDisplay *const disp in m_displays) {
		if ([m_invalidates objectForKey:[disp fileName]] == nil) {
			[m_invalidates setObject:disp forKey:[disp fileName]];
		}
	}
	//NSLog(@"[m_displays count = %d, [m_invalidates count] = %d", [m_displays count], [m_invalidates count]);
	[m_displays removeAllObjects];
	[m_sotedArray release];
	m_sotedArray = nil;
}

/**
 * after add files.
 */
- (void)updateAll
{
	//const NSUInteger cnt = [m_invalidates count];
	//NSLog(@"[m_invalidates count] = %d", cnt);
	[m_invalidates removeAllObjects];
	[self childrenSort];
}

/**
 * set path dictionary.
 * @param[in] pathDictionary path dictionary.
 */
- (void)setPathDictionary:(NSMutableDictionary *)pathDictionary
{
	m_pathDictionary = pathDictionary;
	id fdOld = [m_pathDictionary objectForKey:m_fullPath];
	if (fdOld != nil) {
		NSLog(@"fdOld = %@, self = %@", [fdOld fileName], [self fileName]);
		[m_pathDictionary removeObjectForKey:m_fullPath];
	}
	[m_pathDictionary setObject:self forKey:m_fullPath];
}

/**
 * create from file display.
 * @param[in] fileDisplay object, this object must be a directory.
 * @return newly created object.
 */
+ (FileDisplayCollection *)fileDisplayCollectionWithFileDisplay:(FileDisplay*)fileDisplay
{
	FileDisplayCollection *const fdc = [[FileDisplayCollection alloc] init];
	if (fdc) {
		fdc->m_name = [fileDisplay fileName];
		[fdc->m_name retain];
		fdc->m_date = [fileDisplay fileDate];
		[fdc->m_date retain];
		fdc->m_owner = [fileDisplay fileOwner];
		[fdc->m_owner retain];
		fdc->m_group = [fileDisplay fileGroup];
		[fdc->m_group retain];
		fdc->m_size = [fileDisplay fileSize];
		fdc->m_mode = [fileDisplay fileMode];
		fdc->m_isDirectory = YES;
		fdc->m_isLink = [fileDisplay isLink];
	}
	return fdc;
}

/**
 * data retrieve from NSTableView.
 * @param[in] tableView table view instance.
 * @param[in] tableColumn table column instance.
 * @param[in] row index of the row.
 * @return object to display.
 */
- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
	NSString *const name = [tableColumn identifier];	// identifier is same as English name.
	FileDisplay *const disp = [m_sotedArray objectAtIndex:row];
	if ([name compare:@"Name"] == NSOrderedSame) {
		return [disp fileName];
	}
	else if ([name compare:@"Owner"] == NSOrderedSame) {
		return [disp fileOwner];
	}
	else if ([name compare:@"Group"] == NSOrderedSame) {
		return [disp fileGroup];
	}
	else if ([name compare:@"Date"] == NSOrderedSame) {
		return [disp fileDate];
	}
	else if ([name compare:@"Size"] == NSOrderedSame) {
		return [NSString stringWithFormat:@"%d", [disp fileSize]];
	}
	else if ([name compare:@"Mode"] == NSOrderedSame) {
		//return [disp fileModeString];
		return [NSString stringWithFormat:@"%03d", [disp fileMode]];
	}
	else if ([name compare:@"Icon"] == NSOrderedSame) {
		// show image.
		id addDelegate = [NSApp delegate];
		if ([disp isDirectory]) {
			return [addDelegate folderImage];
		}
		else {
			return [addDelegate documentImage];
		}

	}

	
	return @"";
}

/**
 * tell a NSTableView how many parameters this dialog has.
 * @param[in] tableView table view instance.
 * @return child object count.
 */
- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView
{
	return [m_sotedArray count];
}

/**
 * text cell editing start.
 * @param[in] tableView table view instance.
 * @param[in] tableColumn table column instance.
 * @param[in] rowIndex index of the row.
 * @return YES if not lin NO if link.
 */
- (BOOL)tableView:(NSTableView *)aTableView shouldEditTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
{
	NSLog(@"aTableColumn = %@, rowIndex = %d", [[aTableColumn dataCell] stringValue], rowIndex);
	id ds = [aTableView dataSource];
	id fd = [ds displayObjectAtIndex:rowIndex];
	NSString *const identifier = [aTableColumn identifier];
	[ds setIdentifierTagOnEdit:identifier];
	NSString *val = [[aTableColumn dataCell] stringValue];
	if (([fd isLink]) && ([identifier compare:@"Name"] == NSOrderedSame)) {
		val = [fd fileTitle];
	}
	[ds setColumnValueOnEdit:val];

	return YES;
}

/**
 * text cell editing end.
 * @param[in] tableView table view instance.
 * @param[in] anObject edited value.
 * @param[in] tableColumn table column instance.
 * @param[in] rowIndex index of the row.
 * @return YES.
 */
- (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
{
	//NSLog(@"aTableColumn = %@", [[aTableColumn dataCell] stringValue]);
	//NSLog(@"anObject = %@, rowIndex = %d", anObject, rowIndex);

	NSString *const inputVal = [NSString stringWithFormat:@"%@", anObject];
	if ([[self columnValueOnEdit] compare:inputVal] != NSOrderedSame) {

		id ds = [aTableView dataSource];
		id client = [m_controller client];
		[client setCurrentDirectory:[ds fullPath]];
		if ([@"Name" compare:[self identifierTagOnEdit]] == NSOrderedSame) {
			// try to change name.
			if ([m_controller renameFile:self from:[self columnValueOnEdit] to:inputVal] != NO) {
				// command was success.
				[m_controller setCurrentDirectory:m_fullPath];	// LIST
			}
		}
		else if ([@"Mode" compare:[self identifierTagOnEdit]] == NSOrderedSame) {
			// try to change mode.
			if ([inputVal length] != 3) {
				[MyUtil messageBoxInternational:@"Invalid mode format"];
				return;
			}

			id fd = [m_sotedArray objectAtIndex:rowIndex];
			NSString *const name = [fd fileName];
			[name retain];
			if ([m_controller chmod:self file:[fd fileName] mode:inputVal] != NO) {
				const NSUInteger nMode = [inputVal intValue];
				[fd setFileMode:nMode];
				if ([m_sortKey compare:@"Mode"] == NSOrderedSame) {
					[self childrenSort];
					for (NSUInteger i = 0; i < [m_sotedArray count]; i++) {
						id fd2 = [m_sotedArray objectAtIndex:i];
						if ([name compare:[fd2 fileName]] == NSOrderedSame) {
							NSIndexSet *const indexSet = [NSIndexSet indexSetWithIndex:i];
							[aTableView selectRowIndexes:indexSet byExtendingSelection:NO];
							break;
						}
					}
				}
				// change 
				[aTableView reloadData];
			}
			[name release];
		}
	}
}

/**
 * expands only current directory.
 * @param[in] outlineView Outline view to expand.
 * @param[in] currentDir Current directory.
 */
- (void)expandOutlineView:(NSOutlineView *)outlineView currentDirectory:(NSString *)currentDir
{
	if ([currentDir rangeOfString:m_fullPath].location == 0) {
		[outlineView expandItem:self];
		if ([currentDir compare:m_fullPath] == NSOrderedSame) {
			// current directory.
			NSIndexSet *const indexSet = [NSIndexSet indexSetWithIndex:[outlineView rowForItem:self]];
			[outlineView selectRowIndexes:indexSet byExtendingSelection:NO];
		}
	}
	else {
		//[outlineView collapseItem:self];
	}

	for (NSUInteger i = 0; i < [m_sotedArray count]; i++) {
		id dir = [m_sotedArray objectAtIndex:i];
		if ([dir class] == [FileDisplayCollection class]) {
			[dir expandOutlineView:outlineView currentDirectory:currentDir];
		}
	}
}

/**
 * copy children.
 * @param[in] src copy source.
 */
- (void)copyChildren:(id)src
{
	if (src != nil) {
		for (NSUInteger i = 0; i < [src count]; i++) {
			[m_displays addObject:[src objectAtIndex:i]];
		}
	}
}

/**
 * ask path is exist.
 * @param[in] path path to ask is exist or not.
 * @param[out] isDir YES if path is directory, NO if path is physical file.
 * @return YES if path exists, NO if not.
 */
- (BOOL)hasPath:(NSString *)path isDir:(BOOL*)isDir
{
	// update list.
	//id client = [m_controller client];
	//[self invalidate];
	//[client list:m_fullPath];
	//[self updateAll];

	for (NSUInteger i = 0; i < [m_displays count]; i++) {
		id fd = [m_displays objectAtIndex:i];
		const BOOL dirPath = [fd isDirectory];	// directory path.
		if ([[fd fileName] compare:path] == NSOrderedSame) {
			// name matched.
			if (isDir != NULL) {
				*isDir = dirPath;
			}
			return YES;
		}
		else {
			// name not matched.
			if (dirPath != NO) {	// if directory.
				// starts with same string?
				const NSRange range = [path rangeOfString:[fd fileName]];
				if (range.location == 0) {	// start string is same.
					// still have chance, so go to ask child directory.
					if ([fd internalHasPath:path isDir:isDir intrenalPath:[fd fileName]] != NO) {
						return YES;
					}
				}
			}
		}
	}
	return NO;
}

/**
 * ask path is exist, internal version.
 * @param[in] path path to ask is exist or not.
 * @param[out] isDir YES if path is directory, NO if path is physical file.
 * @param[in] intrenalPath internal built path to ask is exist or not.
 * @return YES if path exists, NO if not.
 */
- (BOOL)internalHasPath:(NSString *)path isDir:(BOOL*)isDir intrenalPath:(NSString *)intrenalPath
{
	// update list.
	id client = [m_controller client];
	[self invalidate];
	[client list:m_fullPath];
	[self updateAll];

	for (NSUInteger i = 0; i < [m_displays count]; i++) {
		id fd = [m_displays objectAtIndex:i];
		NSString *const relPath = [intrenalPath stringByAppendingPathComponent:[fd fileName]];
		const BOOL dirPath = [fd isDirectory];	// directory path.
		if ([relPath compare:path] == NSOrderedSame) {
			// name matched.
			if (isDir != NULL) {
				*isDir = dirPath;
			}
			return YES;
		}
		else {
			// name not matched.
			if (dirPath != NO) {	// if directory.
				// starts with same string?
				const NSRange range = [path rangeOfString:relPath];
				if (range.location == 0) {	// start string is same.
					// still have chance, so go to ask child directory.
					if ([fd internalHasPath:path isDir:isDir intrenalPath:relPath] != NO) {
						return YES;
					}
				}
			}
		}
	}
	return NO;
}

/**
 * get file display.
 * @param[in] pathNames related path name.
 * @return file display if not found nil.
 */
- (id)fileDisplay:(NSString *)pathName
{
	if ([pathName compare:m_name] == NSOrderedSame) {
		return self;
	}
	NSArray *const pathNames = [pathName componentsSeparatedByString:@"/"];
	NSUInteger depth = 0;
	if (([pathNames count] > 0) && ([[pathNames objectAtIndex:0] length] == 0)) {
		depth++;
	}
	return [self fileDisplay:pathNames depth:depth];
}

/**
 * get file display.
 * @param[in] pathNames devided path names of related path.
 * @param[in] depth depth of pathNames.
 * @return file display if not found nil.
 */
- (id)fileDisplay:(NSArray *)pathNames depth:(NSUInteger)depth
{
	NSString *const path = [pathNames objectAtIndex:depth];
	//NSLog(@"path = %@", path);
	for (NSUInteger i = 0; i < [m_displays count]; i++) {
		id fd = [m_displays objectAtIndex:i];
		NSString *const fileName = [fd fileTitle];
		if ([path compare:fileName] == NSOrderedSame) {
			if (depth + 1 == [pathNames count]) {
				return fd;
			}
			else {
				return [fd fileDisplay:pathNames depth:depth + 1];
			}
		}
	}
	return nil;
}

/**
 * accepts drop or not.
 * @param[in] tableView an table view object operation drag and drop.
 * @param[in] info drag and drop information.
 * @param[in] row the index of the row.
 * @param[in] operation the operation type.
 * @return YES if accepts drop, NO if not.
 */
- (BOOL)tableView:(NSTableView *)tableView acceptDrop:(id <NSDraggingInfo>)info
			  row:(int)row dropOperation:(NSTableViewDropOperation)operation;
{
	//NSLog(@"tableView:acceptDrop:row:dropOperation:");
	return [m_controller droppedFromFinder:info fileDisplay:self];
}

/**
 * accepts drop or not into the row.
 * @param[in] tableView an table view object operation drag and drop.
 * @param[in] info drag and drop information.
 * @param[in] row the index of the propsesed row.
 * @param[in] operation the operation type.
 * @return drag operation type.
 */
- (NSDragOperation)tableView:(NSTableView *)tableView validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row
	   proposedDropOperation:(NSTableViewDropOperation)operation;
{
	//NSLog(@"tableView: validateDrop:%@ proposedRow: proposedDropOperation:", info);
	[tableView setNeedsDisplay:YES];
	return NSDragOperationCopy;
}

/**
 * prepare drag operation and copy the file name to the paste board.
 * @param[in] tableView an table view object operation drag and drop.
 * @param[in] row the index of the row to paste.
 * @param[in] pboard paste board object.
 * @return YES to paste NO to not paste.
 */
- (BOOL)tableView:(NSTableView *)tableView writeRows:(NSArray *)rows toPasteboard:(NSPasteboard *)pboard;
{
	NSLog(@"tableView:writeRows:toPasteboard:");
	NSMutableArray *const files = [[NSMutableArray alloc] init];

	//id anObject = nil;
	//NSEnumerator *const enumerator = [[m_displays objectAtIndex:0] objectEnumerator];
	//while (anObject = [enumerator nextObject]) {
		//[files addObject:[anObject valueForKey:@"File"]];
	//	NSLog(@"anObject = %@", anObject);
	//}
	// set file path into paste board.

	[pboard declareTypes:[NSArray arrayWithObject:NSFilenamesPboardType] owner:self];
    [pboard setPropertyList:files forType:NSFilenamesPboardType];

	[files release];

	return YES;
}

/**
 * sort method.
 * @param[in] tableView an table view object operation drag and drop.
 * @param[in] oldDescriptors old sort type.
 */
- (void)tableView:(NSTableView *)tableView sortDescriptorsDidChange:(NSArray *)oldDescriptors;
{
	//NSLog(@"tableView:sortDescriptorsDidChange:");
}

/**
 * double clicked.
 * @param[in] tableView table view object.
 * @param[in] table column object.
 */
- (void)tableView:(NSTableView *)tableView didClickTableColumn:(NSTableColumn *)tableColumn
{
	// Either reverse the sort or change the sorting column
	NSLog(@"tableView:%@ didClickTableColumn:%@", tableView, tableColumn);

	id ds = [tableView dataSource];

	NSString *const name = [tableColumn identifier];	// identifier is same as English name.
	if ([name compare:@"Icon"] == NSOrderedSame) {
		return;	// have no space to show arrow icon.
	}
	if ([name compare:[ds sortKey]] == NSOrderedSame) {
		// change order.
		[ds setAscending:[ds isAscending] == NO ? YES : NO];
		NSString *const imageName = ([ds isAscending] == NO) ? @"NSDescendingSortIndicator" : @"NSAscendingSortIndicator";
		[tableView setIndicatorImage:[NSImage imageNamed:imageName] inTableColumn:tableColumn];
	}
	else {
		// change column.
		[(FileDropTableView *) tableView clearColumnImages];	// clear column image.
		[ds setSortKey:name];
		[ds setAscending:YES];
		NSString *const imageName = @"NSAscendingSortIndicator";
		[tableView setIndicatorImage:[NSImage imageNamed:imageName] inTableColumn:tableColumn];
	}

	// reload.
	[ds childrenSort];
	[tableView reloadData];
}

/**
 * children sort.
 */
- (void)childrenSort
{
	[m_sotedArray release];
	m_sotedArray = [[NSMutableArray alloc] init];
	for (id fd in m_displays) {
		if ([[fd fileName] characterAtIndex:0] == '.') {
			// hidden.
			continue;
		}
		[fd setParentSortKey:m_sortKey];
		[fd setParentAscending:m_ascending];
		[m_sotedArray addObject:fd];
	}
	// children sort.
	[m_sotedArray sortUsingSelector:@selector(compare:)];
}

/**
 * add dummy directory for a while.
 * @param[in] dirName directory name.
 */
- (void)addDummyDirectory:(NSString *)dirName
{
	BOOL found = NO;
	for (NSUInteger i = 0; i < [m_displays count]; i++) {
		id fd = [m_displays objectAtIndex:i];
		if ([dirName compare:[fd fileTitle]] == NSOrderedSame) {
			// no need to add dummy directory.
			found = YES;
			break;
		}
	}
	if (found == NO) {
		FileDisplayCollection *const fdc = [[FileDisplayCollection alloc] init];
		[m_displays addObject:fdc];
		[m_sotedArray addObject:fdc];
		[fdc setFileName:dirName];
		[fdc setFullPath:[m_fullPath stringByAppendingPathComponent:dirName]];
		//[fdc setDummy:YES];
		fdc->m_pathDictionary = m_pathDictionary;
		[m_pathDictionary setObject:fdc forKey:[fdc fullPath]];
	}
}


// constructor.
- (id)init
{
	self = [super init];
	if (self) {
		m_displays = [[NSMutableArray alloc] init];
		m_sotedArray = nil;
		m_invalidates = [[NSMutableDictionary alloc] init];
		[self setIdentifierTagOnEdit:@""];
		[self setColumnValueOnEdit:@""];
		m_sortKey = @"Name";
		[m_sortKey retain];
		m_ascending = YES;
		m_isDirectory = YES;	// of course this is a directory.
		//m_dummy = NO;
	}
	return self;
}

// destructor.
- (void)dealloc
{
	NSLog(@"dealloc = %p", self);
	[m_sortKey release];
	[m_columnValueOnEdit release];
	[m_identifierTagOnEdit release];
	[m_invalidates release];
	[m_sotedArray release];
	[m_displays release];
	[super dealloc];
}

@end
