/**@file
 * MenuEnabler.m
 * @brief Enable/Disable menu.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/10.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "MenuEnabler.h"
#import "MyDocument.h"
#import "ViewController.h"
#import "ShsslAppDelegate.h"
#import "FileDisplayController.h"
#import "ControlDisabler.h"

@implementation MenuEnabler

// properties.
@synthesize m_canExit;

/**
 * get current document.
 * @return current document.
 */
+ (id)currentDocument
{
	NSDocumentController *const docController = [NSDocumentController sharedDocumentController];
	return [docController currentDocument];
}

/**
 * cannot redo enabler.
 * @return YES, if cannot undo, NO can undo.
 */
- (BOOL)cannotUndo
{
	id doc = [MenuEnabler currentDocument];
	if ([doc isDocumentEdited]) {
		// document is edited.
		id controller = [doc controller];
		// if dconnected, you cannot undo.
		return [controller isConnected] ? YES : NO;
	}
	return YES;
}

/**
 * cannot redo enabler.
 * @return YES, if cannot redo, NO can redo.
 */
- (BOOL)cannotRedo
{
	id doc = [MenuEnabler currentDocument];
	if ([doc canRedoCnt] > 0) {
		// document has redo count.
		id controller = [doc controller];
		// if dconnected, you cannot redo.
		return [controller isConnected] ? YES : NO;
	}
	return YES;
}

/**
 * cannot rename enabler.
 * @return YES, if cannot rename and chmod, NO can rename and chmod.
 */
- (BOOL)cannotRename
{
	id doc = [MenuEnabler currentDocument];
	id controller = [doc controller];
	NSUInteger cnt = [controller fileSelectionCount];
	if (cnt == 1) {
		return ([[controller controlDisabler] canClose] == NO) ? YES : NO;
	}
	else {
		return YES;
	}
}

/**
 * cannot remove file enabler.
 * @return YES, if cannot remove, NO can remove.
 */
- (BOOL)cannotRemove
{
	id doc = [MenuEnabler currentDocument];
	id controller = [doc controller];
	NSUInteger cnt = [controller fileSelectionCount];
	if (cnt > 0) {
		return ([[controller controlDisabler] canClose] == NO) ? YES : NO;
	}
	else {
		return YES;
	}
}

/**
 * cannot mkdir enabler.
 * @return YES, if cannot mkdir, NO can mkdir.
 */
- (BOOL)cannotMkdir
{
	id doc = [MenuEnabler currentDocument];
	id controller = [doc controller];
	if ([controller isConnected] == NO) {
		return YES;
	}
	else {
		return ([[controller controlDisabler] canClose] == NO) ? YES : NO;
	}
}

/**
 * cannot paste enabler.
 * @return YES, if cannot paste, NO can paste.
 */
- (BOOL)cannotPaste
{
	id doc = [MenuEnabler currentDocument];
	id controller = [doc controller];
	if ([controller isConnected] == NO) {
		// no connection, no paste.
		return ([[controller controlDisabler] canClose] == NO) ? YES : NO;
	}
	else {
		id delegate = [NSApp delegate];
		const NSUInteger cnt = [delegate copyCount];
		if (cnt > 0) {
			return ([[controller controlDisabler] canClose] == NO) ? YES : NO;
		}
		else {
			return YES;
		}
	}								
}

/**
 * cannot paste from finder enabler.
 * @return YES, if cannot paste, NO can paste.
 */
- (BOOL)cannotPasteFromFinder
{
	id doc = [MenuEnabler currentDocument];
	id controller = [doc controller];
	if ([controller isConnected] == NO) {
		// no connection, no paste.
		return ([[controller controlDisabler] canClose] == NO) ? YES : NO;
	}
	else {
		// see the contnent of paste board.
		NSPasteboard *const pboard = [NSPasteboard generalPasteboard];
		NSArray *const typeArray = [pboard propertyListForType:NSFilenamesPboardType];
		return ([typeArray count] > 0) ? NO : YES;
	}								
}

/**
 * cannot parent directory.
 * @return YES, if cannot parent directory, NO can parent directory.
 */
- (BOOL)cannotParentDirectory
{
	id doc = [MenuEnabler currentDocument];
	id controller = [doc controller];
	if ([controller isConnected] == NO) {
		// no connection, no paste.
		return YES;
	}
	else {
		id fdc = [controller fileDisplayController];
		NSString *const curDir = [fdc currentDirectory];
		if ([curDir compare:@"/"] == NSOrderedSame) {
			return YES;
		}
		else {
			return ([[controller controlDisabler] canClose] == NO) ? YES : NO;
		}
	}								
}

/**
 * cannot close document.
 * @return YES, if cannot close document, NO can close document.
 */
- (BOOL)cannotClose
{
	id doc = [MenuEnabler currentDocument];
	id controller = [doc controller];
	return ([[controller controlDisabler] canClose] == NO) ? YES : NO;
}

/**
 * cannot exit.
 * @return YES, if cannot exit, NO can exit.
 */
- (BOOL)cannotExit
{
	return (m_canExit == NO) ? YES : NO;
}

// constructor.
- (id)init
{
	self = [super init];
	if (self) {
		m_canExit = YES;
	}
	return self;
}

@end
