/**@file
 * ShsslAppDelegate.m
 * @brief Application delegate class.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/18.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "ShsslAppDelegate.h"
#import "CopySource.h"

// constant.
static NSString *const g_shsslType = @"jp.co.comona.shssl.ft";	// shssl file transfer internal type.

@implementation ShsslAppDelegate

// properties.
@synthesize m_folderImage;
@synthesize m_documentImage;
@synthesize m_menuEnabler;

/**
 * copy.
 * @param[in] fileNames file names array.
 * @param[in] isCut YES if cut, NO if copy.
 * @param[in] controller controller adddress.
 */
- (void)copyData:(NSArray *)fileNames isCut:(BOOL)isCut ofController:(id)controller
{
	//NSLog(@"copyData: isCut:%@: controller:%@", (isCut == NO) ? @"NO" : @"YES", controller);

	// calcurate data size.
	NSUInteger dataSize = sizeof(BOOL) + sizeof(NSUInteger);
	for (NSString *const file in fileNames) {
		dataSize += sizeof(NSUInteger) + [[file dataUsingEncoding:NSUTF8StringEncoding] length];
	}
	// create data.
	NSMutableData *const data = [NSMutableData data];
	[data setLength:dataSize];
	// write data.
	unsigned char *internalData = (unsigned char *) [data bytes];
	BOOL *const cut = (BOOL *) internalData;
	*cut = isCut;
	internalData += sizeof(BOOL);
	NSUInteger *sz = (NSUInteger *) internalData;
	*sz = [fileNames count];
	internalData += sizeof(NSUInteger);
	for (NSString *const file in fileNames) {
		sz = (NSUInteger *) internalData;
		NSData *const pathData = [file dataUsingEncoding:NSUTF8StringEncoding];
		const NSUInteger dataLen = [pathData length];
		*sz = dataLen;
		internalData += sizeof(NSUInteger);
		memcpy(internalData, [pathData bytes], dataLen);
		internalData += dataLen;
	}

	// copy into paste board.
	NSPasteboard *const pasteboard = [NSPasteboard generalPasteboard];
	[pasteboard declareTypes:[NSArray arrayWithObject:g_shsslType] owner:self];
	[pasteboard setData:data forType:g_shsslType];

	m_controller = controller;
}

/**
 * empty copy.
 */
- (void)emptyCopy
{
	NSPasteboard *const pasteboard = [NSPasteboard generalPasteboard];
	[pasteboard declareTypes:[NSArray arrayWithObject:g_shsslType] owner:self];
	[pasteboard setData:[NSData data] forType:g_shsslType];	// empty paste board.
	m_controller = nil;
}

/**
 * controller dicconect.
 * @param[in] controller controller adddress.
 */
- (void)controllerDisconnect:(id)controller
{
	//NSLog(@"controllerDisconnect:%@", controller);
	if ((m_controller == controller) || (m_controller == nil)) {
		NSPasteboard *const pasteboard = [NSPasteboard generalPasteboard];
		[pasteboard declareTypes:[NSArray arrayWithObject:g_shsslType] owner:self];
		[pasteboard setData:[NSData data] forType:g_shsslType];	// empty paste board.
		m_controller = nil;
	}
}

/**
 * copy count.
 * @return copy count of internal copy data.
 */
- (NSUInteger)copyCount
{
	NSPasteboard *const pasteboard = [NSPasteboard generalPasteboard];
	NSArray *const typeArray = [pasteboard types];
	if ([typeArray containsObject:g_shsslType]) {
		id data = [pasteboard dataForType:g_shsslType];
		if ((data == nil) || ([data length] == 0)) {
			return 0;
		}
		else {
			return 1;
		}
	}
	else {
		return 0;
	}
}

/**
 * get copy file array.
 * @return copy source file name array.
 */
- (NSArray *)copyFileArray
{
	NSMutableArray *const array = [NSMutableArray array];
	NSPasteboard *const pasteboard = [NSPasteboard generalPasteboard];
	NSArray *const typeArray = [pasteboard types];
	if ([typeArray containsObject:g_shsslType]) {
		NSData *const data = [pasteboard dataForType:g_shsslType];
		if ((data != nil) || ([data length] != 0)) {
			// get cut flag.
			unsigned char *const pData = (unsigned char *) [data bytes];
			unsigned char *internalData = pData;
			const BOOL isCut = *((BOOL *) internalData);
			internalData += sizeof(BOOL);
			// get cout of file name data.
			const NSUInteger cnt = *((NSUInteger *) internalData);
			internalData += sizeof(NSUInteger);
			for (NSUInteger i = 0; i < cnt; i++) {
				// file name length in byte.
				const NSUInteger sz = *((NSUInteger *) internalData);
				internalData += sizeof(NSUInteger);
				// create file name.
				NSData *const fileNameData = [NSData dataWithBytes:internalData length:sz];	// do not use NoCopy version.
				NSString *const fileName = [[NSString alloc] initWithData:fileNameData encoding:NSUTF8StringEncoding];
				CopySource *const cpSrc = [[CopySource alloc] initWithFileName:fileName isCut:isCut sourceController:m_controller];
				[array addObject:cpSrc];
				[cpSrc release];
				[fileName release];
				internalData += sz;
			}
		}
	}
	return array;
}

/**
 * application startup.
 * @param[in] notification notification class.
 */
- (void)applicationDidFinishLaunching:(NSNotification *)notification
{
}

/**
 * application terminate.
 * @param[in] notification notification class.
 */
- (void)applicationWillTerminate:(NSNotification *)notification
{
}

// constructor.
- (id)init
{
	self = [super init];
	if (self) {
		m_folderImage = [[[NSWorkspace sharedWorkspace] iconForFileType:NSFileTypeForHFSTypeCode(kGenericFolderIcon)] retain];
		m_documentImage = [[[NSWorkspace sharedWorkspace] iconForFileType:NSFileTypeForHFSTypeCode(kGenericDocumentIcon)] retain];
		m_controller = nil;
	}
	return self;
}

// destructor.
- (void)dealloc
{
	[m_documentImage release];
	[m_folderImage release];
	[super dealloc];
}

@end
