/**@file
 * XmlResponseScanner.m
 * @brief NSXMLParser do not very well to deal with some Japanase characters.
 * @author toru kageyama
 * Created by Toru Kageyama on 10/10/09.
 * Copyright 2010 Comona Co., Ltd. All rights reserved.
 */

#import "XmlResponseScanner.h"
#import "ViewController.h"
#import "ShsslClient.h"
#import "FileDisplayController.h"
#import "Base64.h"

@implementation XmlResponseScanner

// properties.
@synthesize m_xmlString;
@synthesize m_command;
@synthesize m_resultTarget;
@synthesize m_shsslClient;
@synthesize m_contentReceiver;

/**
 * scan.
 * @return YES if sccess, NO if failed.
 */
- (BOOL)scan
{
	NSScanner *scanner = nil;
	BOOL result = NO;
	@try {
		// login result.
		scanner = [[NSScanner alloc] initWithString:m_xmlString];
		if ([self scanLoginResult:scanner] == NO) {
			[m_shsslClient setConnect:NO];
			@throw [NSException exceptionWithName:@"XML parse" reason:@"Login failed" userInfo:nil];
		}

		if ([self scanCommand:scanner] != NO) {	// command.
			[m_shsslClient setCommandResult:YES];	// command was success.
			[self scanCurrentDirectory:scanner];	// current directory.
			[self scanResult:scanner];	// result contents.
			result = YES;
		}
		else {
			@throw [NSException exceptionWithName:@"XML parse" reason:@"Command failed" userInfo:nil];
		}
	}
	@catch (NSException *e) {
		result = NO;
		@throw e;
	}
	@finally {
		[scanner release];
	}
	return result;
}

/**
 * scan login result.
 * @param[in] scanner NSScanner object.
 * @return YES if sccess, NO if failed.
 */
- (BOOL)scanLoginResult:(NSScanner *)scanner
{
	[scanner scanUpToString:@"<login_result>" intoString:nil];
	NSString *login_result = nil;
	[scanner scanUpToString:@"</login_result>" intoString:&login_result];
	login_result = [login_result substringFromIndex:[@"<login_result>" length]];
	//NSLog(@"login_result = \"%@\"", login_result);

	return ([@"TRUE" compare:login_result] == NSOrderedSame) ? YES : NO;
}

/**
 * scan command.
 * @param[in] scanner NSScanner object.
 * @return YES if sccess, NO if failed.
 */
- (BOOL)scanCommand:(NSScanner *)scanner
{
	[scanner scanUpToString:@"<command>" intoString:nil];
	NSString *command = nil;
	[scanner scanUpToString:@"</command>" intoString:&command];
	command = [command substringFromIndex:[@"<command>" length]];
	[self setCommand:command];

	[scanner scanUpToString:@"<command_result>" intoString:nil];
	NSString *command_result = nil;
	[scanner scanUpToString:@"</command_result>" intoString:&command_result];
	command_result = [command_result substringFromIndex:[@"<command_result>" length]];
	//NSLog(@"command_result = \"%@\"", command_result);
	
	return ([@"TRUE" compare:command_result] == NSOrderedSame) ? YES : NO;
}

/**
 * scan current directory.
 * @param[in] scanner NSScanner object.
 */
- (void)scanCurrentDirectory:(NSScanner *)scanner
{
	[scanner scanUpToString:@"<current_dir>" intoString:nil];
	NSString *current_dir = nil;
	[scanner scanUpToString:@"</current_dir>" intoString:&current_dir];
	current_dir = [current_dir substringFromIndex:[@"<current_dir>" length]];
	//NSLog(@"current_dir = \"%@\"", current_dir);
	[m_resultTarget setCurrentDirectory:current_dir];
}

/**
 * scan result.
 * @param[in] scanner NSScanner object.
 * @return result count.
 */
- (NSUInteger)scanResult:(NSScanner *)scanner
{
	[scanner scanUpToString:@"<result_count>" intoString:nil];
	NSString *result_count = nil;
	[scanner scanUpToString:@"</result_count>" intoString:&result_count];
	result_count = [result_count substringFromIndex:[@"<result_count>" length]];
	//NSLog(@"result_count = \"%@\"", result_count);
	const NSUInteger cnt = [result_count intValue];

	// command result process.
	if (([COMMAND_CONN compare:m_command] == NSOrderedSame) || ([COMMAND_LIST compare:m_command] == NSOrderedSame)) {
		// each paths to examine.
		for (NSUInteger i = 0; i < cnt; i++) {
			[self scanListResult:scanner];
		}
	}
	else if ([COMMAND_RETR compare:m_command] == NSOrderedSame) {
		// RETR command.
		for (NSUInteger i = 0; i < cnt; i++) {
			[self scanRetrResult:scanner];
		}
	}

	return cnt;
}

/**
 * scan LIST result.
 * @param[in] scanner NSScanner object.
 * @return result count.
 * @thow NSException.
 */
- (void)scanListResult:(NSScanner *)scanner
{
	// retrive directory name.
	[scanner scanUpToString:@"<list path=\"" intoString:nil];
	NSString *dir_name = nil;
	[scanner scanUpToString:@"\">" intoString:&dir_name];
	dir_name = [dir_name substringFromIndex:[@"<list path=\"" length]];
	NSLog(@"Processing = \"%@\"", dir_name);
	[[m_resultTarget fileDisplayController] setProcessPath:dir_name];

	// getting path block.
	NSString *pathBlock = nil;
	[scanner scanUpToString:@"</list>" intoString:&pathBlock];
	pathBlock = [pathBlock substringFromIndex:[@"\">" length]];
	//NSLog(@"pathBlock = \"%@\"", pathBlock);

	[self scanPath:pathBlock];
}

/**
 * scan path element.
 * @param[in] pathBlock path block string.
 * @throw NSException
 */
- (void)scanPath:(NSString *)pathBlock
{
	NSScanner *scanner = nil;
	@try {
		scanner = [[NSScanner alloc] initWithString:pathBlock];
	
		while (YES) {
			@try {
				[scanner scanUpToString:@"<path>" intoString:nil];
			}
			@catch (NSException *e) {
				break;
			}
			NSString *path_name = nil;
			[scanner scanUpToString:@"</path>" intoString:&path_name];
			if (path_name == nil) {
				break;
			}
			path_name = [path_name substringFromIndex:[@"<path>" length]];
			if ((path_name == nil) || ([path_name length] < 1)) {
				break;
			}
			//NSLog(@"path_name = \"%@\"", path_name);
			// set path name.
			[[m_resultTarget fileDisplayController] addFileDisplay:path_name];
		}
		// directory finish.
		if ([COMMAND_CONN compare:m_command] == NSOrderedSame) {
			[[m_resultTarget fileDisplayController] addFileDisplayEnd];
		}
	}
	@catch (NSException *e) {
		@throw e;
	}
	@finally {
		[scanner release];
	}
}

/**
 * scan RETR content.
 * @param[in] scanner NSScanner object.
 * @thow NSException.
 */
- (void)scanRetrResult:(NSScanner *)scanner
{
	// getting base64 string.
	[scanner scanUpToString:@"<retr>" intoString:nil];
	NSString *content = nil;
	[scanner scanUpToString:@"\n</retr>" intoString:&content];
	content = [content substringFromIndex:[@"<retr>" length]];
	//NSLog(@"content = \"%@\"", content);
	// convert to original binary.
	NSData *const base64ed = [content dataUsingEncoding:NSUTF8StringEncoding];
	NSData *const origBin = [Base64 decode:base64ed];
	[m_contentReceiver appendData:origBin];
}

// constructor.
- (id)init
{
	self = [super init];
	if (self) {
		m_xmlString = nil;
		m_command = nil;
		m_shsslClient = nil;
		m_resultTarget = nil;
		m_contentReceiver = nil;
	}
	return self;
}

// destructor.
- (void)dealloc
{
	[m_command release];
	[m_xmlString release];
	[super dealloc];
}

@end
